.PHONY: all lib test clean_prj clean_all lint doc $(PRJ_LIBS)

lib: $(PRJ_TARGET_C_OBJ)
	@rm -f $(PRJ_EXE)
	@$(MKDIR) $(PRJ_EXE_DIR)
	@$(MKDIR) $(PRJ_SIZE_DIR)
	$(AR) $(ARFLAGS) $(PRJ_EXE) $(PRJ_TARGET_C_OBJ)
	$(NM) $(NMFLAGS) $(PRJ_EXE) > $(PRJ_SIZE_TARGET)

prog:
	$(PROGRAM) -p m169 -c stk500 -P /dev/ttyACM0 -F -e -U flash:w:$(PRJ_EXE_BIN_TARGET)

exe: $(PRJ_EXE_BIN_TARGET)

test: $(PRJ_TEST_BIN_TARGET)

clean_prj:
	$(RM) $(PRJ_DOC_DIR)
	$(RM) $(PRJ_EXE_DIR)
	$(RM) $(PRJ_OBJ_DIR)
	$(RM) $(PRJ_MAP_DIR)
	$(RM) $(PRJ_TST_EXE_DIR)

clean_all:
	$(RM) $(ROOT_DIR)/release/execute
	$(RM) $(ROOT_DIR)/release/map
	$(RM) $(ROOT_DIR)/release/lst
	$(RM) $(ROOT_DIR)/release/doc/generate
	$(RM) $(ROOT_DIR)/release/object
	$(RM) $(ROOT_DIR)/release/size

lint:
	$(LINT) $(LNTFLAGS) $(PRJ_TARGET_C_SRC) $(PRJ_TEST_C_SRC) 

doc:
	@$(MKDIR) $(PRJ_DOC_DIR)
	cd $(PRJ_SRC_CORE_DIR)
	(cat $(DOXYFILE) ; echo "INPUT=$(PRJ_DOC_IN_DIR)" ; echo "OUTPUT_DIRECTORY=$(PRJ_DOC_DIR)") | $(DOXYGEN) -

# build dependency files
define makedep
	$(CC) -MM            \
           -MF $3         \
           -MP            \
           -MT $2         \
           $(CFLAGS)      \
           $1
endef

# target to objcopy elf to executable (i.e. hex)
$(PRJ_EXE_BIN_TARGET): $(PRJ_EXE_ELF_TARGET)
	@rm -f $(PRJ_EXE_BIN_TARGET)
	@$(MKDIR) $(PRJ_LST_DIR)
	$(OBJCOPY) $(PRJ_EXE_ELF_TARGET) -O$(FORMAT) $(PRJ_EXE_BIN_TARGET)
	$(OBJDUMP) -S $(PRJ_EXE_ELF_TARGET) > $(PRJ_LST_TARGET)

# target to objcopy elf to eeprom file
$(PRJ_EXE_EEP_TARGET): $(PRJ_EXE_ELF_TARGET)
	$(OBJCOPY) $(EEFLAGS) -O$(FORMAT) $< $@
	
# target to create the elf file
$(PRJ_EXE_ELF_TARGET): $(PRJ_LIBS) $(PRJ_TARGET_C_OBJ)
	@rm -f $(PRJ_EXE_ELF_TARGET)
	$(MKDIR) $(PRJ_EXE_DIR)
	$(MKDIR) $(PRJ_MAP_DIR)
	$(MKDIR) $(PRJ_SIZE_DIR)
	$(CC) $(PRJ_TARGET_C_OBJ) $(PRJ_LIB_EXE) $(LDFLAGS) -o$(PRJ_EXE_ELF_TARGET)
	$(SIZE) $(PRJ_EXE_ELF_TARGET) > $(PRJ_SIZE_TARGET)

$(PRJ_LIBS):
	$(MAKE) lib -C $@\

$(PRJ_TEST_BIN_TARGET): $(PRJ_TARGET_C_OBJ) $(PRJ_TEST_C_OBJ)
	@$(MKDIR) $(PRJ_TST_EXE_DIR)
	$(NATIVE_CC) -o $@ $^ $(LDFLAGS) $(PRJ_TEST_LIBS)

$(PRJ_TARGET_C_OBJ): $(PRJ_OBJ_DIR)/%o : $(PRJ_SRC_CORE_DIR)/%c
	@$(MKDIR) $(PRJ_OBJ_DIR)
	@$(call makedep,$<,$@,$(subst .o,.d,$@))
	$(CC) -c $(CFLAGS) $< -o $@

$(PRJ_TEST_C_OBJ): $(PRJ_OBJ_DIR)/%o : $(PRJ_SRC_TEST_DIR)/%c
	@$(MKDIR) $(PRJ_OBJ_DIR)
	@$(call makedep,$<,$@,$(subst .o,.d,$@))
	$(NATIVE_CC) -c $(CFLAGS) $< -o $@
