/*
 * update.c
 *
 *  Created on: Nov 5, 2011
 *      Author: tkl
 */

#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <string.h>

#include <avr/boot.h>
#include <avr/interrupt.h>

#include "radio.h"
#include "update.h"

#include "uart.h"

#include <util/delay.h>

//-----------------------------------------------------------------------------
static uint8_t sequence_number;
static uint8_t m_page_buffer[SPM_PAGESIZE] = {0xFF};    // buffer to write a complete flash page
static uint8_t m_page_buffer_index = 0;
static bool do_update = true;

//-----------------------------------------------------------------------------
static void send_msg(uint8_t app, uint8_t fkt, bool byte_set, uint8_t byte);
static void handle_ihex_frame();
static void program_page(uint32_t page);

//-----------------------------------------------------------------------------
void start_update(void) {
    uint8_t rx_bytes = 0;
    sequence_number = 0;

    init_uart();

    while(do_update) {
        do {
        	send_msg(APP_UPDATE, FKT_READY_FOR_TASK, false, 0);
        	rx_bytes = rx_packet(rx_buffer, RX_BUFFER_SIZE, 500000);
        }
        while(rx_bytes == 0);
        if((rx_buffer[5] == APP_UPDATE) && (rx_buffer[6] == FKT_NEW_FRAME)) {
        	handle_ihex_frame();
        }
    }
}

//-----------------------------------------------------------------------------
static void send_msg(uint8_t app, uint8_t fkt, bool byte_set, uint8_t byte) {
    uint8_t buffer[8];

    if(byte_set) {
        buffer[0] = 8;
        buffer[7] = byte;
    }
    else {
        buffer[0] = 7;
    }
    buffer[1] = 0x01;//eeprom_read_byte(MST_ADDR_EE_ADDR);
    buffer[2] = 0x02;//eeprom_read_byte(DEV_ADDR_EE_ADDR);
    buffer[3] = sequence_number++;
    buffer[4] = 0x00;
    buffer[5] = app;
    buffer[6] = fkt;
    tx_packet(buffer, buffer[0]);
}

//-----------------------------------------------------------------------------
static void handle_ihex_frame() {
#if 0
    uint16_t calc_chk = 0;
    for(uint8_t count = 1; count <= (len - 1); count++) {
        calc_chk += buffer[count];
    }
    calc_chk &= 0x00FF;

    if(calc_chk != 0) {
        // TODO: invalid frame - add error handling
        return;
    }
#endif
    uint16_t	record_addr = (rx_buffer[8] << 8) + rx_buffer[9];
    uint8_t		record_type = rx_buffer[10];
    uint16_t	flash_page_addr = record_addr - record_addr % SPM_PAGESIZE;

    switch(record_type) {
    case 0x00:  // data record
        for(uint8_t count = 11; count < rx_buffer[0]; count++) {
            m_page_buffer[m_page_buffer_index++] = rx_buffer[count];
            if(m_page_buffer_index == SPM_PAGESIZE) {
                // page ready to write
                _delay_ms(100);

                program_page(flash_page_addr);
                memset(m_page_buffer, 0xFF, sizeof(m_page_buffer_index));
                m_page_buffer_index = 0;
            }
        }
        break;

    case 0x01:  // end of file record
        do_update = false;
        break;
    }
    send_msg(APP_UPDATE, FKT_NEW_FRAME_PROCESSED, true, rx_buffer[3]);
}
//-----------------------------------------------------------------------------
static void program_page(uint32_t page) {

    uint8_t sreg;
    // Disable interrupts
    sreg = SREG;
    cli();

    eeprom_busy_wait ();
    boot_page_erase (page);
    boot_spm_busy_wait ();              // Wait until the memory is erased.
    for (uint16_t i = 0; i < SPM_PAGESIZE; i += 2) {
        uint16_t low = (uint16_t)m_page_buffer[i];
        uint16_t high = (uint16_t)(m_page_buffer[i + 1] & 0x00FF);
        high *= 256;

        uint32_t lw = (high + low);
        uint16_t w = (uint16_t) lw;
       boot_page_fill (page + i, w);
    }
    boot_page_write (page);             // Store buffer in flash page.
    boot_spm_busy_wait();               // Wait until the memory is written.

    // Reenable RWW-section again. We need this if we want to jump back to the
    // application after bootloading.
    boot_rww_enable();

    // Re-enable interrupts (if they were ever enabled).
    SREG = sreg;
}
